//
#include "keyboard.h"
#include "hardware.h"	/*        */

enum 
{
	KBD_STABIL_PRESS =			NTICKS(50),	//   ()   
	KBD_STABIL_RELEASE =		NTICKS(50),	//   ()   

	KBD_MAX_PRESS_DELAY_LONG =	NTICKS(400), //        
	KBD_PRESS_REPEAT_SLOW =		NTICKS(80),	//      

	KBD_MAX_PRESS_DELAY_SHORT = NTICKS(200), //      c  
	KBD_PRESS_REPEAT_QUICK =	NTICKS(10)	//      
};

#if CPUSTYLE_ATMEGA
static uint_fast8_t kbd_press;	/*     */
static uint_fast8_t kbd_release;		/*    -    */
#else
static uint_fast16_t kbd_press;	/*     */
static uint_fast16_t kbd_release;		/*    -    */
#endif /* CPUSTYLE_ATMEGA */

static uint8_t kbd_last;	/*  - (   ) */

/* keyboard functions */

static uint8_t 
is_slow_repeat(uint8_t key)
{
	//return memchr(autoreplist, key, sizeof autoreplist) != NULL;
	switch (key)
	{
	/*   ,        */
	case KBD_CODE_BUTTON1:	//   
	case KBD_CODE_BUTTON2:	//   
		return 1;
	}
	return 0;
}

static uint8_t 
is_fast_repeat(uint8_t key)
{
	//return memchr(autoreplist, key, sizeof autoreplist) != NULL;
	switch (key)
	{
	/*   ,        */
	//case KBD_CODE_STEP_DOWN:	//   
	//case KBD_CODE_STEP_UP:	//   
	//	return 1;
	}
	return 0;
}

/*  -   0   .
 *   ,  -    0x80
 * 
 */
static uint8_t
kbd_scan_local(uint8_t * key)
{
	uint8_t chinp;
	const int notstab = (kbd_press < (KBD_STABIL_PRESS + 1));

	if (hardware_ged_pressed_key(& chinp))
	{
		if (/*kbd_release != 0 && */ ! notstab && (kbd_last != chinp))		//      
		{
			//   
			kbd_last = chinp;
			kbd_press = 1;
			kbd_release = 0;
			//kbd_slowcount = 0;
			//dbg_putchar('t');
			return 0;
		}
		else if (kbd_release != 0 && ! notstab)
		{
			//     -   .
			// kbd_last    
			//    - .
			//dbg_putchar('k');
		}
		else if (kbd_press == 0)
		{
			//   
			kbd_last = chinp;
			//DBG_HEX8(chinp); dbg_putchar('!');
			kbd_press = 1;
			kbd_release = 0;
			return 0;
		}	
		//else if (notstab && (kbd_last != chinp))		//    
		else if ((kbd_last != chinp))		//    
		{	
			kbd_last = chinp;
			//DBG_HEX8(chinp); dbg_putchar('+');
			kbd_press = 1;
			kbd_release = 0;
			return 0;
		}

		kbd_release = KBD_STABIL_RELEASE;
		
		//DBG_HEX16(kbd_press); dbg_putchar('/');DBG_HEX8(kbd_last); dbg_putchar('/'); DBG_HEX8(chinp); 
		//dbg_putchar(notstab ? '.' : ':');

		if (is_slow_repeat(kbd_last))
		{
			//      
			if (kbd_press < (KBD_MAX_PRESS_DELAY_LONG + KBD_PRESS_REPEAT_SLOW))
			{
				switch (++ kbd_press)
				{
				case KBD_MAX_PRESS_DELAY_LONG + KBD_PRESS_REPEAT_SLOW:
					kbd_press = KBD_MAX_PRESS_DELAY_LONG;	//     .
				case KBD_MAX_PRESS_DELAY_LONG:
					* key = kbd_last;
					//DBG_HEX8(* key); dbg_putchar('U');
					kbd_release = 0;
					return 1;
				}
			}
			return 0;
		}
		else if (is_fast_repeat(kbd_last))
		{
			//      
			if (kbd_press < (KBD_MAX_PRESS_DELAY_SHORT + KBD_PRESS_REPEAT_QUICK))
			{
				switch (++ kbd_press)
				{
				case KBD_MAX_PRESS_DELAY_SHORT + KBD_PRESS_REPEAT_QUICK:
					kbd_press = KBD_MAX_PRESS_DELAY_SHORT;	//     .
				case KBD_MAX_PRESS_DELAY_SHORT:
					* key = kbd_last;
					return 1;
				}
			}
			return 0;
		}
		else
		{
			//      
			if (kbd_press == KBD_MAX_PRESS_DELAY_LONG)
				return 0;	// lond_press symbol already returned
			if (kbd_press < KBD_MAX_PRESS_DELAY_LONG)
			{
				if (++ kbd_press == KBD_MAX_PRESS_DELAY_LONG)
				{	
					* key = (kbd_last | KBD_LONGPRESS); // lond_press symbol
					return 1;
				}
			}
		}
		return 0;

	}
	else if (kbd_release != 0) //    -  
	{
		if (notstab)
		{
			kbd_press = 0;		//    
			kbd_release = 0;
			return 0;
		}
		//dbg_putchar('r');
		// keyboard keys released, time is not expire.
		if (-- kbd_release == 0)
		{
			// time is expire
			if (is_fast_repeat(kbd_last) && kbd_press < KBD_MAX_PRESS_DELAY_SHORT)
			{
				* key = kbd_last;	
				kbd_press = 0;
				return 1;		//      .
			}
			else if (kbd_press < KBD_MAX_PRESS_DELAY_LONG)
			{
				* key = kbd_last;	
				kbd_press = 0;
				return 1;		//      .
			}
			else
			{
				kbd_press = 0;
				return 0;		//         .
			}
		}
		return 0;

	}
	else //        
		return 0;
}

static volatile uint_fast8_t kbd_scancode;
static volatile uint_fast8_t kbd_repeat;

#define EVENT_PERIOD 200	// ms   S-  SWR-

#if CPUSTYLE_ATMEGA
static volatile uint_fast8_t timer_0p1_s_repeat;
#else
static volatile uint_fast16_t timer_0p1_s_repeat;
#endif /* CPUSTYLE_ATMEGA */

// Called from Timer 0 overflow interrupt service routine
//   TICKS_FREQUENCY 
void
kbd_spool(void)
{
	uint8_t code;
	if (kbd_scan_local(& code) != 0)
	{
		//if (code != KBD_CODE_BW_AGC)
		//{
		//	dbg_puts_impl("UPS!\n");
		//	for (;;)
		//		;
		//}
		if (kbd_scancode == code)
			++ kbd_repeat;
		else
		{
			kbd_scancode = code;
			kbd_repeat = 1;
		}
	}
	//
	if (timer_0p1_s_repeat < NTICKS(EVENT_PERIOD))
	{
		++ timer_0p1_s_repeat;
	}

}
/*  -   0   .
 *   ,  -    0x80
 */
uint_fast8_t kbd_scan(uint_fast8_t * key, uint_fast8_t * repeat)
{
	uint_fast8_t r;
	disableIRQ();
	if (kbd_scancode != 0)
	{
		* key = kbd_scancode;
		* repeat = kbd_repeat;
		kbd_scancode = 0;
		r = 1;
	}
	else
	{
		r = 0;
	}
	enableIRQ();
	return r;
}

/* ,    
 */
uint_fast8_t kbd_get_ishold(uint_fast8_t key)
{
	uint_fast8_t r;
	disableIRQ();
	r = (kbd_press != 0) && ((kbd_last & 0x7f) == key);
	enableIRQ();
	return r;
}
#if WITHBARS

uint_fast8_t 
timer_getevent(void)
{
	uint_fast8_t r;
	disableIRQ();
	if (timer_0p1_s_repeat >= NTICKS(EVENT_PERIOD))
	{
		r = 1;
		timer_0p1_s_repeat = 0;
	}
	enableIRQ();
	return r;
}
#endif

/*      */
/*  - -       setup */
void kbd_initialize(void)
{

	kbd_press = 0;
	kbd_release = 0;
	//harware_kbd_initialize(); moved to main
	kbd_scancode = 0;

	//kbd_state = KBDSTATE_IDLE;

	// .   .
	timer_0p1_s_repeat = 0;

	//dbg_puts_impl("KBD_STABIL_PRESS = "); DBG_HEX16(KBD_STABIL_PRESS); dbg_puts_impl("\n");
	//dbg_puts_impl("KBD_STABIL_RELEASE = "); DBG_HEX16(KBD_STABIL_RELEASE); dbg_puts_impl("\n");

}


void kbd_wait_initialize(void)
{

	for (;;)
	{

	}
}
