#include "hardware.h"
#include "keyboard.h"


/*      low..high  "" */
/*      */
static uint_least8_t
calc_next(uint_least8_t v, uint_least8_t low, uint_least8_t high)
{
	if (v < low || v >= high)
		return low;
	return v + 1;
}
/*      low..high  "" */
/*      */
static uint_least8_t 
calc_prev(uint_least8_t v, uint_least8_t low, uint_least8_t high)
{
	if (v <= low || v > high)
		return high;
	return v - 1;
}

static volatile uint_fast8_t rxflag;
static volatile uint_fast8_t rxcode;

static uint_fast8_t position;
static uint_fast8_t oldpos;			//   
//static uint_fast8_t maxposition;	// 8  16 -  
enum { maxposition = 16 };			//  16 

//       UART
//    
void cat_parsechar(uint_fast8_t c)	
{
	if ((c & 0xf0) != 0xf0)
		return;
	
	rxcode = c & 0x0f;
	rxflag = 1;

}

//      
static uint_fast8_t getrxcode(uint_fast8_t * v)
{
	uint_fast8_t f;

	disableIRQ();

	* v = rxcode;
	f = rxflag;
	rxflag = 0;
	enableIRQ();

	return f;
}

/*   -  - , 
	      
	*/
static uint_least8_t
processkeyboard(void)
{
#if 0
	// demo cycles
	_delay_ms(500);
	position = calc_next(position, 0, maxposition - 1);
	return 1;
#elif 1
	if (getrxcode(& position))
		return 1;
	return 0;
#else

	uint_least8_t kbch, repeat;

	if (kbd_scan(& kbch, & repeat) != 0)
	{
		//ledrefresh();

		switch (kbch)
		{
		case KBD_CODE_BUTTON1:
			while (repeat --)
				position = calc_next(position, 0, maxposition - 1);
			return 1;	//   

		case KBD_CODE_BUTTON1 | KBD_LONGPRESS:
			return 0;	//    

		case KBD_CODE_BUTTON2:
			while (repeat --)
				position = calc_prev(position, 0, maxposition - 1);
			return 1;	//   

		case KBD_CODE_BUTTON2 | KBD_LONGPRESS:
			return 0;	//    

		default:
			break;
		}
	}
	return 0;	//    
#endif
}

void board_relay_set(
	uint_fast8_t i, //  
	uint_fast8_t state)	// 0 - ,  - 
{
	if (state == 0)
	{
		// 
		switch (i)
		{
		case 0:
			PORTC |= (1 << PC0); break;
		case 1:
			PORTC |= (1 << PC1); break;
		case 2:
			PORTC |= (1 << PC2); break;
		case 3:
			PORTC |= (1 << PC3); break;
		case 4:
			PORTC |= (1 << PC4); break;
		case 5:
			PORTC |= (1 << PC5); break;

		case 6:
			PORTB |= (1 << PB0); break;
		case 7:
			PORTB |= (1 << PB1); break;

		case 8:
			//PORTD |= (1 << PD0); break;		
			PORTB |= (1 << PB2); break;	//   ,     8/16

		case 9:
			PORTD |= (1 << PD1); break;
		case 10:
			PORTD |= (1 << PD2); break;
		case 11:
			PORTD |= (1 << PD3); break;
		case 12:
			PORTD |= (1 << PD4); break;
		case 13:
			PORTD |= (1 << PD5); break;
		case 14:
			PORTD |= (1 << PD6); break;
		case 15:
			PORTD |= (1 << PD7); break;
		}
	}
	else
	{
		// 
		switch (i)
		{
		case 0:
			PORTC &= ~ (1 << PC0); break;
		case 1:
			PORTC &= ~ (1 << PC1); break;
		case 2:
			PORTC &= ~ (1 << PC2); break;
		case 3:
			PORTC &= ~ (1 << PC3); break;
		case 4:
			PORTC &= ~ (1 << PC4); break;
		case 5:
			PORTC &= ~ (1 << PC5); break;

		case 6:
			PORTB &= ~ (1 << PB0); break;
		case 7:
			PORTB &= ~ (1 << PB1); break;

		case 8:
			//PORTD &= ~ (1 << PD0); break;	
			PORTB &= ~ (1 << PB2); break;		//   ,     8/16
		case 9:
			PORTD &= ~ (1 << PD1); break;
		case 10:
			PORTD &= ~ (1 << PD2); break;
		case 11:
			PORTD &= ~ (1 << PD3); break;
		case 12:
			PORTD &= ~ (1 << PD4); break;
		case 13:
			PORTD &= ~ (1 << PD5); break;
		case 14:
			PORTD &= ~ (1 << PD6); break;
		case 15:
			PORTD &= ~ (1 << PD7); break;
		}
	}


}


//     

void board_relay_initialize(void)
{
	PORTD |= 0xfe;	// PD7..PD1 as output, initial state = 1
	DDRD |= 0xfe;

	PORTB |= 0x07;	// PB2, PB1..PB0 as output, initial state = 1
	DDRB |= 0x07;

	PORTC |= 0x3f;	// PC5..PC0 as output, initial state = 1
	DDRC |= 0x3f;
}



void updateboard(uint_fast8_t force)
{
	if (force != 0 || oldpos != position)
	{
		board_relay_set(oldpos, 0);
		_delay_ms(10);
		board_relay_set(position, 1);
		_delay_ms(10);
	}
}

int main(void)
{
	hardware_timer_initialize();

	board_relay_initialize();
	hardware_adc_initialize();
	//hardware_cat_initialize(1);
	//hardware_cat_set_speed(9600);
	{

		// USART initialization
		// Communication Parameters: 8 Data, 1 Stop, No Parity
		// USART Receiver: On
		// USART Transmitter: Off
		// USART Mode: Asynchronous
		// USART Baud Rate: 9600
		UCSRA=0x00;
		UCSRB=0x90;
		UCSRC=0x86;
		UBRRH=0x00;
		UBRRL=0x33;
	}

	kbd_initialize();

	enableIRQ();

	while (hardware_adc_started() == 0)
		;
	_delay_ms(20);
	_delay_ms(20);
	_delay_ms(20);
	_delay_ms(20);

	position = 0;
	//maxposition = (PINB & (1U << PB2)) != 0 ? 16 : 8;
	updateboard(1);	//     

	for (;;)
	{
		oldpos = position;
		if (processkeyboard())
		{
			//    -   
			updateboard(0);

		} // end keyboard processing
	}
	return 0;
}
